<?php
require_once('widgets.inc.php');

class BookshelvesController extends WidgetsController {

	var $name = 'Bookshelves';
	var $helpers = array('Html', 'Form', 'BookHelper', 'Javascript','Ajax', 'Time');
	var $components = array('RequestHandler', 'Auth', 'Security');
	var $user_id = null;
	var $uses = array('BookshelvesBooks','Bookshelf');
	/**
	 * @var $Bookshelf
	 */
	var $Bookshelf;
	/**
	 * @var $BookshelvesBooks
	 */
	var $BookshelvesBooks;
	
	function beforeFilter() {
		parent::beforeFilter();
		$this->user_id = $this->Auth->user('IdUsager');
		$this->Auth->allow('public_list', 'public_shelf');
	}
	
	function _checkOwner($id) {
		return $this->user_id == array_shift($this->Bookshelf->find('list', array('conditions'=>array($this->Bookshelf->primaryKey => $id), 'fields'=>'user_id')));
	}
    
	
	function index() {
		$this->Bookshelf->recursive = 0;
		$conditions = array('user_id'=>$this->user_id);
		
		$bookshelves = $this->Bookshelf->find('all', compact('conditions'));
		if (empty($bookshelves)) {
			$bookshelves = $this->Bookshelf->createFirstBookshelf($this->user_id);
		}
		
		$id = $bookshelves[0]['Bookshelf'][$this->Bookshelf->primaryKey];
		if (!in_array(LIBER_PLUGIN_WEB20,Configure::read(LIBER_PLUGINS))){
			$this->redirect(array('action'=>'view', 'id'=>$id));
		}
		
		$this->set('bookshelves', $this->paginate($this->Bookshelf,$conditions));
	}
	
	function view($id = null) {
		if (!$id || !$this->_checkOwner($id)) {
			$this->Session->setFlash(__('Неправильная книжная полка.', true));
			$this->redirect(array('action'=>'index'));
		}
		$bookshelf = $this->Bookshelf->read(null, $id);
		$this->set('bookshelf', $bookshelf);
		$books_params = $this->BookshelvesBooks->findAllByBookshelf_id($id);
		
		if (count($books_params) == 0) return;
		
		foreach ($books_params as $book_params){
			$b = $book_params['BookshelvesBooks'];
			$sources[$b['source']][] = $b['notice_id'];
		}

		$books = array();
		$this->Bookshelf->Book->recursive = 1;
		$this->paginate['Book'] = array('order'=>array('Book.IdNotice'));
		$this->paginate['Book']['contain'] = array('NoticeZ', 'ExemplaireStat','Exemplaire'=>a('Statut'),'Rating');
		
		foreach ($sources as $source=>$ids){
			if(!$this->isSourceExist($source)) continue;
			
			$this->Bookshelf->Book->setDataSource($source);
			$books_res = $this->paginate($this->Bookshelf->Book, array('Book.IdNotice'=>$ids));
			foreach ($books_res as $book){
				array_push($books,$book);
			}
		}
		$this->set('books', $books);
	}

	function add() {
		if (!empty($this->data) && !$this->_checkOwner($id)) {
			$this->data['Bookshelf']['user_id'] = $this->user_id;
			$this->data['Bookshelf']['created'] = null;
			//основная полка может быть только одна
			if($this->data['Bookshelf']['is_default'] > 0) {
				$this->Bookshelf->resetDefault($this->user_id);
			}
			$this->Bookshelf->create();
			if ($this->Bookshelf->save($this->data)) {
				$this->Session->setFlash(__('Книжная полка сохранена', true));
				$this->redirect(array('action'=>'index'));
			} else {
				$this->Session->setFlash(__('Книжная полка не может быть сохранена. Пожалуйста повторите попытку.', true));
			}
		}
		//$books = $this->Bookshelf->Book->find('list');
		//$this->set(compact('books'));
	}

	function edit($id = null) {
		if (!$id && empty($this->data) || !$this->_checkOwner($id)) {
			$this->Session->setFlash(__('Неправильная книжная полка.', true));
			$this->redirect(array('action'=>'index'));
		}
		if (!empty($this->data)) {
			$this->Bookshelf->recursve = 0;
			$data = $this->Bookshelf->read(null, $id);
			unset($data['Book']);
			$data = array_merge($this->data,$data);
			//основная полка может быть только одна
			if($this->data['Bookshelf']['is_default'] > 0) {
				$this->Bookshelf->resetDefault($this->user_id);
			}
			if ($this->Bookshelf->save($this->data)) {
				$this->Session->setFlash(__('Книжная полка сохранена', true));
				$this->redirect(array('action'=>'index'));
			} else {
				$this->Session->setFlash(__('Книжная полка не может быть сохранена. Пожалуйста повторите попытку.', true));
			}
		}
		if (empty($this->data)) {
			$this->data = $this->Bookshelf->read(null, $id);
		}
		//$conditions = array('user_id'=>$this->user_id);
		//$books = $this->Bookshelf->Book->find('list');
		$this->set(compact('books'));
	}

	function delete($id = null) {
		if (!$id || !$this->_checkOwner($id)) {
			$this->Session->setFlash(__('Неверный номер для книжной полки', true));
			$this->redirect(array('action'=>'index'));
		}
		if ($this->Bookshelf->del($id, false)) {
			$this->Session->setFlash(__('Книжная полка удалена', true));
			$this->redirect(array('action'=>'index'));
		}
	}
	
	function remove($id = null) {
		if (!$id || !$this->_checkOwner($id)) {
			$this->Session->setFlash(__('Неверный номер для книжной полки', true));
			$this->redirect(array('action'=>'view'));
		}
		if (!$this->params['named']['IdNotice']) {
			$this->Session->setFlash(__('Неверный номер для книги', true));
			$this->redirect(array('action'=>'view'));
		}
		
		$id = intval($id);
		$notice_id = intval($this->params['named']['IdNotice']);
		$source = Sanitize::paranoid($this->params['named']['Source']);
		
		$conditions = array('bookshelf_id'=>$id, 'notice_id'=>$notice_id, 'source'=>$source);
		$this->BookshelvesBooks->deleteAll($conditions);
		//$this->Bookshelf->query("DELETE FROM bookshelves_books WHERE bookshelf_id = $id AND notice_id = $notice_id");
		
		if ($this->Bookshelf->getAffectedRows()) {
			$this->Session->setFlash(__('Книга удалена с полки', true));
			$this->redirect(array('action'=>'view', $id));
		} else {
			$this->Session->setFlash(__('Не возможно удалить книгу с полки', true));
			$this->redirect(array('action'=>'view', $id));
		}
	}
	
	function print_shelf($id = null){
		$this->layout = 'ajax';
		
		if(!$id){
			$id = intval($this->params['form']['id']);
		}
		if(!$id || !$this->_checkOwner($id)){
			$this->Session->setFlash(__('Неверный номер для книжной полки', true));
			$this->redirect(array('action'=>'view'));
		}
		$books = $this->BookshelvesBooks->findAllByBookshelf_id($id);
		
		$sources = Set::combine($books,'/BookshelvesBooks/notice_id','/BookshelvesBooks/notice_id','/BookshelvesBooks/source');
		foreach ($sources as $source=>$ids){
			$sources[$source] = implode(',',$ids);
		}
		$res = $this->requestAction(array('controller' => 'Books', 'action' => 'print_books'), array('return', 'form' => $sources));
		$this->set('res',$res);
	}
	
	 
    function public_list(){
    	if (!in_array(LIBER_PLUGIN_WEB20, Configure::read(LIBER_PLUGINS))){
			$this->redirect('/') ;	
		}
    	$query = str_replace('*','%',$this->params['data']['Bookshelf']['libelle']);
    	
    	$condition = array('Bookshelf.is_public >'=> 0);
    	if(is_string($query) && $query != ''){
    		$condition['Bookshelf.libelle like'] = $query; 
    	}
    	
    	$this->paginate['Bookshelf']['order'] = array('Bookshelf.libelle','Bookshelf.created');
    	
    	$this->Bookshelf->recursive = 0;
    	$data = $this->paginate($this->Bookshelf,$condition);
    	$this->set('bookshelves',$data);
    }
    
    function public_shelf($id=0){
    	if (!in_array(LIBER_PLUGIN_WEB20, Configure::read(LIBER_PLUGINS))){
			$this->redirect('/') ;	
		}
    	// проверка доступа
    	$this->Bookshelf->recursive = 2;
    	$bookshelf = $this->paginate($this->Bookshelf,array('id'=>$id));
    	//$bookshelf = $this->Bookshelf->read(null, $id);
		$this->set('bookshelf', $bookshelf[0]['Bookshelf']);
		$this->set('books', $bookshelf[0]['Book']);
    }

}
?>