<?php
require_once('widgets.inc.php');

class CommentsController extends WidgetsController {

	var $name = 'Comment';

	var $uses = array('Comment', 'Book');
	var $helpers = array('Html', 'Form', 'BookHelper');
	
	var $user_id = null;
	var $notice_id = null;

	function beforeFilter(){
		if (!in_array(LIBER_PLUGIN_WEB20, Configure::read(LIBER_PLUGINS))){
			$this->redirect('/') ;	
		}
		parent::beforeFilter();
		$this->user_id = $this->Auth->user('IdUsager');
		$this->Auth->allow('view');
		
		if($id = $this->data['Comment']['notice_id']){
			$this->notice_id = $id;
		}
		
		header('Cache-Control: no-cache');
	}
	
	function _checkOwner($idComment = null) {
		$cond = array(
			'conditions'=>array('user_id' => $this->user_id, 'notice_id' => $this->notice_id), 
			'fields'=>array('user_id')
		);
		if(is_numeric($idComment) && $idComment !== null){
			$cond['conditions']['id'] = $idComment;
		}
		$res = $this->Comment->find('list',$cond);
		return ($res != false && count($res)>0);
	}
	
	function add(){
		if (!empty($this->data)) {
			$this->data['Comment']['user_id'] = $this->user_id;
			$this->data['Comment']['created'] = null;
			if(trim($this->data['Comment']['comment']) == ''){
				$this->Session->setFlash(__('Пустой комментарий недопустим.', true));
			}else{
				$this->Comment->create();
				$message = ($this->Comment->save($this->data)) ? __('Комментарий сохранен', true)
					: __('Комментарий не может быть сохранена. Пожалуйста повторите попытку.', true);
				$this->Session->setFlash($message);
			}
		}
		$this->redirect(array('controller'=>'notices','action'=>'index',$this->notice_id));	
	}
	
	function view($idNotice = null){
		if($idNotice != null){
			$this->notice_id = $idNotice;
		}
		$comments = $this->Comment->find('all',array('conditions'=>array('notice_id'=>$idNotice)));
		
		// вставка возможности изменить/удалить комментарий
		if($this->_checkOwner()){
			foreach ($comments as $nom=>$comment){
				$comments[$nom]['Comment']['is_modified'] = ($this->user_id == $comment['Comment']['user_id']);
			}
		}
		
		if (isset($this->params['requested'])) {
			return $comments;
		}else{
			$this->set(compact('comments'));
		}
	}
	
	function delete($idComment){
		$comment = $this->Comment->find('first',array('conditions'=>array('id'=>$idComment)));
		$this->notice_id = $comment['Comment']['notice_id'];
		
		if(!$this->notice_id) {
			$this->Session->setFlash(__('Не задан документ', true));
		}
		
		if(!$this->_checkOwner($idComment)){
			$this->Session->setFlash(__('Нельзя удалить этот комментарий', true));
		}else{
			if($this->Comment->delete($idComment)){
				$this->Session->setFlash(__('Комментарий успешно удален', true));
			}else{
				$this->Session->setFlash(__('Ошибка удаления комментария', true));
			}
		}
		$this->redirect(array('controller'=>'notices','action'=>'index',$this->notice_id));
	}
	
	function edit(){
		if(!$this->_checkOwner()){
			// not action but notice
		}
	}

}
?>