<?php
class RatingsController extends AppController {

	var $name = 'Ratings';
    var $helpers = array('Html','Ajax','Javascript');
    var $components = array( 'RequestHandler', 'Security');
    var $uses = array('Rating', 'Ratelog');
	
	/**
	 * @var Ratelog
	 */
	var $Ratelog;
	/**
	 * @var Rating
	 */
	var $Rating;
	
	/**
	 * @var RequestHandlerComponent
	 */
	var $RequestHandler;
	// префикс для cookies
	var $_prefix = 'id_notice_voted_';
	
	function beforeFilter () {
		$this->Auth->allow(array('index', 'view', 'add'));
		
		$this->paginate = array(
          'limit' => 25,
          'order' => array(
              'Ratings.rating' => 'desc',
          )
        );
    }
    
    function beforeRender () {
    	parent::beforeRender();
    	if ($this->RequestHandler->isAjax()) {
			  $this->layout = 'ajax';
			  Configure::write('debug', 0);
		}
    }

	function index() {
		$this->Rating->recursive = 0;
		$this->set('ratings', $this->paginate());
	}

	function view($id = null) {
		if (!$id) {
			$this->Session->setFlash(__('Неправильный рейтинг.', true));
			$this->redirect(array('action'=>'index'));
		}
		$this->set('rating', $this->Rating->read(null, $id));
	}

	function add() {
		if (!empty($this->params['form'])) {
			
			$data = $this->params['form'];
			$notice_id = array_pop(array_keys($data['star']));
			$rating    = array_pop(array_values($data['star']));
			
			if($this->_checkAdd($notice_id)){
				$result = $this->Rating->inc(compact('notice_id','rating'));
				if ($result) {
					$this->_setCookie($result);
					$this->_addToLog($result);
					$this->set('result', $result);
				} else {
					$this->set('error', __('Не удалось учесть Ваш голос. Пожауйста, попробуйте еще раз позже.', true));
				}
			}else{
				$this->set('error', __('Нет прав проставления оценки или вы уже голосовали', true));
			}
		}
	}
	
	function _setCookie($vote){
		$this->Cookie->write($this->_prefix.$vote['Rating']['notice_id'],'ok');
	}
	
	function _checkAdd($notice_id){
		// проверка на права добавления рейтинга и повторного голосования
		return $this->_hasPermission() && !$this->_isVoted($notice_id);
	}
	
	function _hasPermission(){
		$has_perm = false;
		switch (Configure::read('opac.voting.permission')){
			case 'reg':
				if($this->Auth->user('IdUsager') > 0) $has_perm = true;
				break;
			case 'all':
				 $has_perm = true;
				 break;
			case 'none':
			default:
				 $has_perm = false;
				 break;
		}
		return $has_perm;
	}
	
	function _isVoted($notice_id){
		return ($this->Cookie->read($this->_prefix.$notice_id) != null) ;
	}
	
	function _addToLog($vote){
		if(!(Configure::read('opac.voting.log') == 'on')) return;
		
		$params = is_array($vote['Rating']) ? $vote['Rating'] : array();
		$this->Ratelog->set(array(
					'user_id' => ($this->Auth->user('IdUsager')) ? $this->Auth->user('IdUsager') : 0,
					'rating_id' => $params['notice_id'],
					'remote_addrs' => $_SERVER['REMOTE_ADDR'],
					'remote_host' => $_SERVER['REMOTE_HOST'],
					'user_agent' => $_SERVER['HTTP_USER_AGENT'],
					'viewed' => '1'
				));
		$this->Ratelog->save();
	}

	function edit($id = null) {
		if (!$id && empty($this->data)) {
			$this->Session->setFlash(__('Неправильный рейтинг.', true));
			$this->redirect(array('action'=>'index'));
		}
		if (!empty($this->data)) {
			if ($this->Rating->save($this->data)) {
				$this->Session->setFlash(__('Рейтинг был сохранён', true));
				$this->redirect(array('action'=>'index'));
			} else {
				$this->Session->setFlash(__('Рейтинг не может быть сохранён. Пожалуйста попробуйте ещё раз.', true));
			}
		}
		if (empty($this->data)) {
			$this->data = $this->Rating->read(null, $id);
		}
	}

	function delete($id = null) {
		if (!$id) {
			$this->Session->setFlash(__('Неправильный id для рейтинга', true));
			$this->redirect(array('action'=>'index'));
		}
		if ($this->Rating->del($id)) {
			$this->Session->setFlash(__('Рейтинг удалён', true));
			$this->redirect(array('action'=>'index'));
		}
	}

}
?>