<?php
class Exemplaire extends AppModel {

	var $name = 'Exemplaire';
	var $useTable = 'EXEMPLAIRE';
	var $primaryKey = 'IdExemplaire';
	var $actsAs = array('Containable');
	var $cacheQueries = true;

	//The Associations below have been created with all possible keys, those that are not needed can be removed
	var $belongsTo = array(
		'Notice' => array(
			'className' => 'Notice',
			'foreignKey' => 'idNotice',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		),
		'Book' => array(
			'className' => 'Book',
			'foreignKey' => 'idNotice',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		),
		'Annexe' => array(
			'className' => 'Annexe',
			'foreignKey' => 'idAnnexe',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		),
		'Fond' => array(
			'className' => 'Fond',
			'foreignKey' => 'IdFonds',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		),
		'Bibliotheque' => array(
			'className' => 'Bibliotheque',
			'foreignKey' => 'idBibliotheque',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		),
		'Support' => array(
			'className' => 'Support',
			'foreignKey' => 'IdSupport',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		),
		'Statut' => array(
			'className' => 'Statutexemplaire',
			'foreignKey' => 'IdStatut',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		)
	);
	
	
	/**
	 * Вычисляет поле "Возможность выдачи", isPretable, для каждого экземпляра
	 * @todo fix and use associations
	 * @param array $exemplaire
	 * @return boolean
	 */
	function _getIsPretable($exemp) {
		return (boolean)(
		empty($exemp['IdReservation'])
		&& empty($exemp['IdPret'])
		&& ($exemp['PhasePretable'] !== 0)
		&& ($exemp['StatutPretable'] !== 0));
	}
	
	/*
	function afterFind($results, $primary = false) {
		// add isPretable field to exemplaire
		foreach ($results as $index => $row) {
				$results[$index][$this->alias]['idPretable'] = $this->_getIsPretable($row[$this->alias]);
		}
		return $results;
	}*/
	
	/**
	 * Находит id описания для заданного id экземпляра в БД
	 *
	 * @param integer $id = IdExemplaire = id экземпляра
	 * @return integer    = IdNotice     = id описания
	 */
	function getIdNoticeByIdExmplaire($id){
		$tableName = $this->name;
		$cond = array(
			'conditions' => array("$tableName.idExemplaire" => $idExmp),
			'fields' => array("$tableName.idNotice"),
		);
		$result = $this->find('first',$cond);
		
		if(count($result) == 0){
			return false;
		}
			
		return $result["$tableName"]['idNotice'];
	}
	
	
	/**
	 * @example test it on 
	 * SELECT TOP 10 IdNotice, count(DISTINCT IdAnnexe), count(DISTINCT IdFonds) FROM EXEMPLAIRE 
	 *   GROUP BY IdNotice 
	 *   HAVING count(DISTINCT IdFonds) > 2 
	 *   ORDER BY count(DISTINCT IdAnnexe) DESC
	 * @param integer $IdNotice
	 * @param booblean $grouped
	 * @param string $type - one of 'all', 'preted','reserved'
	 */
	
	function findAllByIdNotice($IdNotice, $grouped = true, $type = 'all') {
		$conditions = array('IdNotice'=>$IdNotice);
		$contain = array('Annexe', 'Fond');
		$order =   array('Annexe.Libelle');
		$joins   = array();
		$joins = array(array('table'=>'PRET', 'type'=>'LEFT', 'conditions'=>'Exemplaire.IdExemplaire = pret.IdExemplaire'),
		               array('table'=>'EXEMPLAIRE_RESA', 'type'=>'LEFT', 'conditions'=>'Exemplaire.IdExemplaire = EXEMPLAIRE_RESA.IdExemplaire'),
		               array('table'=>'STATUTEXEMPLAIRE as Statut', 'type'=>'LEFT', 'conditions'=>'Exemplaire.IdStatut = Statut.IdStatut'));
	    
		if ($grouped) {
			$fields = array('COUNT(Exemplaire.IdExemplaire) AS total', 
							'COUNT(PRET.IdExemplaire) AS preted', 
							'COUNT(EXEMPLAIRE_RESA.IdExemplaire) AS reserved',
							'SUM(Statut.Reservable) AS Reservable',
							"SUM(cast(dbo.IsOPACRaservationAvailable($IdNotice) as int)) as isOpacReservable",
							'Annexe.IdAnnexe', 
							'Annexe.Libelle', 
							'Fond.IdFonds', 
							'Fond.Libelle'
							);
			$group  = array('Annexe.IdAnnexe', 'Annexe.Libelle', 'Fond.IdFonds', 'Fond.Libelle');
			$result = $this->find('all', compact('contain', 'conditions', 'fields', 'group','order', 'joins'));
			return $result;
		} else {
			return $this->find('all', compact('contain', 'conditions', 'order'));
		}
	}
	
	
	/**
	 * @example test it on 
	 * SELECT TOP 10 IdNotice, count(DISTINCT IdAnnexe), count(DISTINCT IdFonds) FROM EXEMPLAIRE 
	 *   GROUP BY IdNotice 
	 *   HAVING count(DISTINCT IdFonds) > 2 
	 *   ORDER BY count(DISTINCT IdAnnexe) DESC
	 * @param integer $IdNotice
	 * @param booblean $grouped
	 * @param string $type - one of 'all', 'preted','reserved'
	 */
	function findAllByIdNotices($aIdNotices) {
		$conditions = array('Exemplaire.IdNotice'=>$aIdNotices);
		$joins   = array();
		$joins = array(array('table'=>'PRET', 'type'=>'LEFT', 'conditions'=>'Exemplaire.IdExemplaire = pret.IdExemplaire'),
		array('table'=>'STATUTEXEMPLAIRE', 'type'=>'LEFT', 'conditions'=>'Exemplaire.IdStatut = STATUTEXEMPLAIRE.IdStatut AND STATUTEXEMPLAIRE.Pretable=0'),
		array('table'=>'EXEMPLAIRE_RESA', 'type'=>'LEFT', 'conditions'=>'Exemplaire.IdExemplaire = EXEMPLAIRE_RESA.IdExemplaire'));


		$fields = array('Exemplaire.IdNotice',
		'COUNT(Exemplaire.IdExemplaire) AS total',
		'(COUNT(Exemplaire.IdExemplaire) - COUNT(PRET.IdExemplaire) - COUNT(EXEMPLAIRE_RESA.IdExemplaire) - COUNT(STATUTEXEMPLAIRE.IdStatut)) AS availible'
		);
		$group  = array('Exemplaire.IdNotice');
		$result = $this->find('all', compact('contain', 'conditions', 'fields', 'group','order', 'joins'));
		return $result;
	}
	
	function findPretable(){
		$conditions = array('IdNotice'=>$IdNotice);
		$fields = array('COUNT(Exemplaire.IdExemplaire) AS total', 
							'COUNT(PRET.IdExemplaire) AS preted', 
							'COUNT(STATUTEXEMPLAIRE.IdStatut) AS unpreteble', 
							'COUNT(EXEMPLAIRE_RESA.IdExemplaire) AS reserved'
							);
		$joins = array(array('table'=>'PRET', 'type'=>'LEFT', 'conditions'=>'Exemplaire.IdExemplaire = pret.IdExemplaire'));
		
		return $this->find('all', compact('conditions', 'fields','joins'));
	}
	
	/**
	 * Возвращает незабронированный штрихкод
	 *
	 * @param int $idNotice id описания
	 * @param int $idAnnexe id филиала
	 * @return int штрихкод экземпляра
	 */
	function findUnreservedExemplaire($idNotice, $idAnnexe){
		
		return $this->query("select top 1 idexemplaire from exemplaire where idnotice = $idNotice and idannexe = $idAnnexe and idexemplaire not in (
		select e.idexemplaire from exemplaire e inner join exemplaire_resa er on e.idexemplaire = er.idexemplaire
		where idnotice = $idNotice)");
	}

}
?>