<?php

class Solr extends AppModel
{
	var $name = 'Solr';
	var $useTable = false;
	var $servers = array(
	//	'default'=> array(
			'host' => '127.0.0.1',
			'port' => '8080',
			'home' => '/solr'	 
	//	),
	);
	var $limit = 10;
	var $page = 0;
	var $sort = array();
	var $request = array('version' => '2.2');
	var $spechalchars = array(' OR ', '*', '~', '(', ')', ':', '-');
	var $fields = array(
		'searcheble' => array('title', 'titles', 'author', 'authors', 'editors',
			'cities', 'years', 'ISBN', 'series', 'rubrics'),
		'system' => array('id','categ'),
		'sortable' => array('title', 'author', 'editors', 'years'),
	);
	
	function simpleSearch($req){
		if($this->_in_str($req,$this->spechalchars) !== false){
			return $this->findEverywhere($req);
		}else{
			return $this->findEverywhereRange($req);
		}
	}
	
	/**
	 * Поиск одной строкой (веса полей дифференцированы)
	 * т.е. title^10 author^15 и т.п.
	 * @param str $req - запрос
	 * @return array ответ Solr
	 */
	function findEverywhereRange($req){
		$url = $this->getUrl().'&qt=dismax'.'&q='.$this->parseRequestFull($req);
		return $this->_send($url);
	}
	
	/**
	 * Поиск одной строкой (веса полей одинаковы)
	 * @param str $req - запрос
	 * @return array ответ Solr
	 */
	function findEverywhere($req){
		$url = $this->getUrl().'&q='.$this->parseRequest($req);
		return $this->_send($url);
	}
	
	function parseRequestFull($req){
		// удалить все кроме слов
		//$req = Sanitize::clean($req);
		return $this->parseRequest($req);
	}
	
	function parseRequest($req){
		$req = Sanitize::stripAll($req);
		$req = mb_strtolower($req);
		$req = str_replace(array(' or ',' или '), ' OR ', $req);
		return urlencode($req);
	}
	
	/**
	 * Прототип ф-ции поиска по заданному полю
	 *
	 * @param srt $fieldName имя поля
	 * @param srt $request	поисковый запрос
	 */
	function findBy($fieldName,$request){
		unset($this->request['query']);
		$this->addField($fieldName,$request);
		$this->findInSolr();
	}
	
	function addField($fieldName,$request){
		if(! in_array($fieldName,$fields['searcheble'])){
			$fieldName = $this->request['searcheble'][0];
		}
		$this->request['query'][$fieldName] = $this->parseRequest($request);
	}
	
	function findInSolr(){
		$url = $this->getUrl();
		foreach ($this->request['query'] as $field=>$request){
			$url .= "&$field=".$request;
		}
		return $this->_send($url);
	}
	
	function setSort($sort){
		if($sort != false){
			list($field,$order) = explode('+',$sort);
			$field = strtolower($field);
			if(in_array($field,$this->fields['sortable'])){
				$order =  ($order=='desc') ? 'desc' : 'asc';
				$this->sort[] = $field.'Sort'.'+'.$order;
			}
		}
	}
	
	function setLimit($num){
		$num = intval($num);
		if($num > 5 && $num <=100){
			$this->limit = $num;
		}else{
			$this->limit = 10;
		}
	}
	
	function setPage($page){
		$page = intval($page-1);
		if($page > 0){
			$this->page = $page;
		}else{
			$this->page = 0;
		}
	}
	
	function getBaseUrl(){
		return 'http://'.$this->servers['host'].':'.$this->servers['port'].$this->servers['home'];
	}
	
	function getUrl(){
		$url = $this->getBaseUrl().'/select?fl=*';
		if($this->request['version']) {
			$url .= '&version='.$this->request['version'];
		}
		$url .= '&start='.$this->page*$this->limit.'&rows='.$this->limit;
		
		if(is_array($this->sort) && count($this->sort) > 0){
			$url .= '&sort='.implode(',',$this->sort);
		}
		return $url;
	}
	
	function _send($url){
		$url .= '&wt=phps';
		if(!extension_loaded('curl')){
			if(!dl("php_curl.dll")){
				return false;
			}
		}
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		$data = curl_exec($ch);
		
		if (curl_errno($ch)) {
			return false;
		}
		
		curl_close($ch);
		
		return unserialize($data);
	}
	
	function reindex($delta=true){
		($delta) ? $this->_deltaImport() : $this->_fullImport();
	}
	
	function _fullImport(){
		$url =$this->getBaseUrl().'/select?qt=%2Fdataimport&verbose=true&clean=true&commit=true&command=full-import';
		$this->_send($url);
	}
	
	function _deltaImport(){
		$url =$this->getBaseUrl().'/select?clean=false&commit=true&qt=%2Fdataimport&command=delta-import';
		$this->_send($url);
	}
	
	function getStatus(){
		$url =$this->getBaseUrl().'/select?clean=false&commit=true&qt=%2Fdataimport&command=status';
		return $this->_send($url);
	}
	
	function _in_str($haystack,$needle=array()){
		foreach ($needle as $need){
			if (strpos($haystack,$need) !== false) return true;
		}
		return false;
	}
}
?>