<?php
/* SVN FILE: $Id: text.php 8283 2009-08-03 20:49:17Z gwoo $ */
/**
 * Book Helper
 *
 * Book description manipulations and output: 
 *   extract, combine, format description fields and values 
 *   from key-value pairs as they are stored in Absotheque NOTICE_Z table
 *
 * PHP versions 4 and 5
 *
 * CakePHP(tm) :  Rapid Development Framework (http://www.cakephp.org)
 * Copyright 2005-2008, Cake Software Foundation, Inc. (http://www.cakefoundation.org)
 *
 * Licensed under The MIT License
 * Redistributions of files must retain the above copyright notice.
 *
 * @filesource
 * @copyright     Copyright 2005-2008, Liber Company (http://www.libermedia.ru)
 * @link          http://www.cakefoundation.org/projects/info/cakephp CakePHP(tm) Project
 * @package       cake
 * @subpackage    cake.cake.libs.view.helpers
 * @since         CakePHP(tm) v 0.10.0.1076
 * @version       $Revision: 8283 $
 * @modifiedby    $LastChangedBy: isor $
 * @lastmodified  $Date: 2009-08-12 13:49:17 +0300 (Wen, 12 Aug 2009) $
 * @license       http://www.opensource.org/licenses/mit-license.php The MIT License
 */
/**
 * Included libraries.
 *
 */
if (!class_exists('Multibyte')) {
	App::import('Core', 'Multibyte');
}
if (!class_exists('TextHelper')) {
	App::import('Helper', 'Text');
}
/**
 * Text helper library.
 *
 * Text manipulations: Highlight, excerpt, truncate, strip of links, convert email addresses to mailto: links...
 *
 * @package       cake
 * @subpackage    cake.cake.libs.view.helpers
 */
class BookHelperHelper extends AppHelper {
	
	/**
	 * @var TextHelper
	 */
	var $text;
	
	function __construct() {
		parent::__construct();
		$this->text = new TextHelper();
		$this->type = array(
		'1' =>	array('name' => __('Книга',true),
								  'img_name'=> 'book'),
		'4'	=>	array('name' => __('Периодическое издание',true),
								   'img_name'=> 'periodic'),
		'5'	=>	array('name' => __('Выпуск',true),
									'img_name'=> 'issue'),
		'6'	=>	array('name' => __('Статья',true),
									'img_name'=> 'article'),
		'7' =>	array('name' => __('Книга (аналит. описание)',true),
									'img_name'=> 'book_analit'),
		'9'	=>	array('name' => __('Многотомник',true),
									'img_name'=> 'multivolume'),
		'10' =>	array('name' => __('Смешанный комплект',true),
									'img_name'=> 'mixed_set'),
		'12' =>	array('name' => __('Ноты',true),
									'img_name'=> 'notes'),
		'13' =>	array('name' => __('Ноты  (аналит. описание)',true),
									'img_name'=> 'notes_analit'),
		'14' =>	array('name' => __('Картографическое издание',true),
									'img_name'=> 'map'),
		'18' =>	array('name' => __('Видеозапись',true),
									'img_name'=> 'video'),
		'19' =>	array('name' => __('Видеозапись  (аналит. описание)',true),
									'img_name'=> 'video_analit'),
		'22' =>	array('name' => __('Микроформа',true),
									'img_name'=> 'microform'),
		'23' =>	array('name' => __('Микрофиша',true),
									'img_name'=> 'microfiche'),
		'24' =>	array('name' => __('Диафильм',true),
									'img_name'=> 'diapositive'),
		'26' =>	array('name' => __('Звукозапись',true),
									'img_name'=> 'record'),
		'27' =>	array('name' => __('Звукозапись (аналит. описание)',true),
									'img_name'=> 'record_analit'),
		'28' =>	array('name' => __('Старопечатное издание',true),
									'img_name'=> 'oldprinted'),
		'29' =>	array('name' => __('Старопечатное издание ( аналит. описание)',true),
									'img_name'=> 'oldprinted_analit'),
		'30' =>	array('name' => __('Электронный ресурс',true),
									'img_name'=> 'electronic'),
		'31' =>	array('name' => __('Электронный ресурс  (аналит. описание)',true),
									'img_name'=> 'electronic_analit'),
		'32' =>	array('name' => __('Игры',true),
									'img_name'=> 'game'),
		'33' =>	array('name' => __('Игры (аналит. описание)',true),
									'img_name'=> 'game_analit'),
		'34' =>	array('name' => __('Сайт Интернет',true),
									'img_name'=> 'site'),
		'35' =>	array('name' => __('Автореферат/Диссертация',true),
									'img_name'=> 'annotation'),
		'44' =>	array('name' => __('Диссертация',true),
									'img_name'=> 'annotation')
		);
	}
	
	function getId($book) {
		if (isset($book['Book'])) return $book['Book']['IdNotice'];
		if (isset($book['IdNotice'])) return $book['IdNotice'];
	}
	
	function value($book, $idZone = '200', $sZone = 'a', $options = array()) {
		$xpath = "/NoticeZ[IdZone=$idZone][sZone=$sZone]";
		return $this->_output($book, $xpath, $options);
	}
	
	function title($book, $options = array()) {
		$aTitle = array();
		
		$options = array_merge(array('truncate' => 100, 'delimiter' => ' '),
			$options);
		$xpath = '/NoticeZ[IdZone=200][sZone=a]';
		$title = $this->_output($book,$xpath,$options);
		
		$aTitle[] = $title;
		$aTitle[] = $this->part_title($book);
		
		$aTitle = Set::filter($aTitle);
		
		if (!empty($aTitle)) return implode('. ', $aTitle);
	}
	
	function source_title($book, $options = array()){
		if (isset($book['Source'][0])) {
			return $this->title($book['Source'][0], $options);
		}
	}
	
	function ptitle($book, $options = array()) {
		$options = array_merge(array('truncate' => 100, 'delimiter' => ' '),
			$options);
		$xpath = '/NoticeZ[IdZone=200][sZone=d]';
		return $this->_output($book,$xpath,$options);
	}
	
	
	/**
	 * Enter description here...
	 * 
	 * Если $options['returnEmpty'] == true то возвращает пустую строку для коллективных авторов
	 * (такое поведение нужно для вывода в заголовках страниц описаний)
	 *
	 * @param array $book
	 * @param array $options
	 * @return mixed
	 */
	function author($book, $options = array()) {
		$options = array_merge(array('returnEmpty'=>false), $options);
		$xpath = '/NoticeZ[IdZone=200][sZone=f]';
		$auteur = $this->_output($book, $xpath, $options);
		if ($auteur == 'Collectif' || empty($auteur))
		  $auteur = ($options['returnEmpty']) ? '' : __('Коллективный автор', true); 
		return $auteur;
	}
	
	function authorAsLink($book, $options = array()) {
		$options = array_merge(array('controller' => 'auteurs'), $options);
		$xpath = '/NoticeZ[IdZone=200][sZone=f]';
		return $auteur = $this->_linkOutput($book, $xpath, $options);
	}
	
	function authors($book, $options = array()) {
		$xpath = '/NoticeZ[IdZone=200][sZone=g]/Valeur';
		return $this->_output($book, $xpath, $options);
	}
	
	function date($book, $options = array()) {
		$xpath = '/NoticeZ[IdZone=210][sZone=d]';
		return $this->_output($book, $xpath, $options);
	}
	
	function editorial($book, $options = array()) {
		$xpath = '/NoticeZ[IdZone=210][sZone=c]';
		return $this->_output($book, $xpath, $options);
	}
	
	function editorialAsLink($book, $options = array()){
		$options = array_merge(array('controller' => 'editeurs'), $options);
		$xpath = '/NoticeZ[IdZone=210][sZone=c]';
		return $collection = $this->_linkOutput($book,$xpath,$options);
	}
	
	function place($book, $options = array()) {
		return $editorial = implode(', ',Set::extract('/NoticeZ[IdZone=210][sZone=a]/Valeur', $book));
	}
	
	function collection($book, $options = array()) {
		$xpath = '/NoticeZ[IdZone=225][sZone=a]';
		return $collection = $this->_output($book,$xpath,$options);
	}
	
	function collectionAsLink($book, $options = array()){
		$options = array_merge(array('controller' => 'collections'), $options);
		$xpath = '/NoticeZ[IdZone=225][sZone=a]';
		return $collection = $this->_linkOutput($book,$xpath,$options);
	}
	
	function isbn($book, $options = array()) {
		$xpath = '/NoticeZ[IdZone=10][sZone=a]';
		return $this->_output($book, $xpath, $options);
	}
	
	function volume($book, $options = array() ){
		$xpath = '/NoticeZ[IdZone=461][sZone=v]';
		$volume1 = $this->_output($book, $xpath, $options);
		$xpath = '/NoticeZ[IdZone=200][sZone=v]';
		$volume2 = $this->_output($book, $xpath, $options);
		return coalesce($volume1,$volume2,$this->part($book,$options),'');
	}
	
	function part($book, $options = array()){
		$xpath = '/NoticeZ[IdZone=200][sZone=h]';
		return $this->_output($book, $xpath, $options);
		 
	}
	
	function part_title($book, $options = array()){
		$xpath = '/NoticeZ[IdZone=200][sZone=i]';
		return $this->_output($book, $xpath, $options);
		 
	}
	
	function doc_type($book,$categ=false){
		if(!$categ) $categ = Set::extract('Book.Categ',$book);
		return $this->type[$categ]['name'];
	}
	
	function img_name($book,$categ=false){
		if(!$categ) $categ = Set::extract('Book.Categ',$book);
		//if(!in_array($categ,$this->type)){
		if(!array_key_exists($categ,$this->type)){
			$categ = 1;
		}
		return $this->type[$categ]['img_name'];
	}
	
	function part_volume($book, $options = array()){
		$volume = $this->volume($book);
		$part   = $this->part($book);
		if (empty($volume) || empty($part)) return coalesce($volume, $part);
		
		if(strpos($this->volume($book),$this->part($book)) !== false){
			return $this->volume($book,$options);
		}elseif(!$this->part($book)){ 
			return $this->volume($book,$options);
		}else{
			return $this->volume($book).', '.$this->part($book,$options);
		}
	}
	
	function subtitle($book, $options = array()){
		$xpath = '/NoticeZ[IdZone=200][sZone=e]';
		return $this->_output($book, $xpath, $options);
	}
	
	function issue($book, $options = array()){
		$xpath = '/NoticeZ[IdZone=207][sZone=a]';
		return $this->_output($book, $xpath, $options);
	}
	
	function full_title($book,$options = array()){
		$part_volume = $this->part_volume($book,array('append'=>'. '));
		return 
		     $this->source_title($book, array('append'=>'. '))
		    .$part_volume                                 // Обозначение тома и-или части
			.$this->title($book, (empty($part_volume) ? array() : array('prepend'=>': '))) // Основное заглавие
			.$this->subtitle($book,array('prepend'=>': ')) // Подзаголовок
			.$this->issue($book,array('prepend'=>' '));    // Номер выпуска
		
	}
	
	function exmp_location($exmp){
		
		$out_str = '';
		$max_len = 100;
		$lib_info = Configure::read('opac.book.exmp_loc_info');
		if( (!is_array($lib_info)) && (count($lib_info)==0) ) return $out_str;
		
		if ( is_array($exmp) && count($exmp)>0){
			$libs = Set::combine($exmp,'/Bibliotheque/IdBibliotheque','/Bibliotheque/Libelle');
			$fils = Set::combine($exmp,'/Annexe/IdAnnexe','/Annexe/Libelle','/Annexe/idBibliotheque');
			
			if(!is_array($libs)) return $out_str;
			
			foreach ($libs as $lib_nom=>$lib_name){
				$s_libs = $lib_name;
				$s_fils = implode(', ',$fils[$lib_nom]);
				
				if(in_array('Bibliotheque',$lib_info) && in_array('Annexe',$lib_info)){
					$str = ($s_libs != '' && $s_fils != '') ? $s_libs.' : '.$s_fils : $s_libs.$s_fils;
				}elseif (in_array('Bibliotheque',$lib_info)){
					$str = $s_libs; 
				}elseif (in_array('Annexe',$lib_info)){
					$str = $s_fils; 
				}
				
				if(strlen($str) > $max_len) $str = substr($str,0,$max_len-3).'...';
				if($str != '') $out_str .= $str.'<br />';
			}
		}
		
		return $out_str;
	}
	
	function exmp_stat($book){
		$out = '';
		$exmp_all = '';
		//$count_exmp = count(Set::extract('/Exemplaire/IdExemplaire',$book));
                $count_exmp = $book['ExemplaireStat']['countAll'];
		//$pretable = $this->requestAction("/books/numberOfExemplaires/".$this->getId($book));
                $pretable = $book['ExemplaireStat']['Pretable'];
		//$reservable = array_sum(Set::extract('/Exemplaire/Statut/Reservable',$book));
                $reservable = $book['ExemplaireStat']['Reservable'];
		
		if($count_exmp <= 0){
			return '<span class="noexmp">'.__('Нет экз.',true).'</span>';
		}elseif ($pretable <= 0){
			$out .= '<span class="noavailable">'.__('Недоступно',true).'</span><br><span class="exmpcount">&nbsp;'.$pretable.' из '.$count_exmp.'</span>';
		}else{
			$out .= '<span class="exmpavailable">'.__('Доступно',true).'</span><br><span class="exmpcount">&nbsp;'.$pretable.' из '.$count_exmp.'</span>';
		}
		return $out;
	}
	
	/**
	 * Возвращает ссылку(ки) на серию, издательство и тд.
	 * @author Pavel
	 * @param array $book
	 * @param xpath $xpath
	 * @param array $options
	 * @return string
	 */
	function _linkOutput($book, $xpath, $options = array()){
		$options = array_merge(array(
					'prepend'    => '',
					'append'     => '',
					'id'         => 'Valeurn',
					'controller' => '',
					'action'     => 'view',
					'source'     => 'default',
					'output'     => 'string',
					'class'      => ''
		),
		$options);
		
		$linkText = $this->_output($book, $xpath, array('output' => 'array'));
		$linkId = $this->_output($book, $xpath, array('field' => $options['id'], 'output' => 'array'));
		
		if(empty($options['controller']) || empty($options['action']) || in_array($linkText[0], array('Collectif','Anonyme','Anonyme','Anonīms','Anonymous')))
			return $this->_output($book, $xpath, $options);
		
		if(!(empty($linkText) || empty($linkId)))
		{
			$html = new HtmlHelper();
			
			$i = 0;
			foreach($linkText as $lt)
			{
				$lt = str_replace('&quot;', '"', $lt);
				$l = $html->link(
					$lt,
					array(
						'controller' => $options['controller'],
						'action' => $options['action'],
						'id' => $linkId[$i],
						'source' => $options['source']
					),
					array('title' => $lt, 'class' => $options['class'])
					);
				
				if($options['output'] == 'array')
					$link[] = $options['prepend'].$l.$options['append'];
				else
					$link[] = $l;
				
				$i++;
			}
			if($options['output'] == 'array')
				return $link;
			else
				return $options['prepend'].implode(', ',$link).$options['append'];
		} else {
			return '';
		}
	}

	function _output($data, $xpath, $options = array()) {
		$options = array_merge(array(
			'truncate' => 0,
			'prepend'  => '',
			'append'   => '',
			'delimiter'=> ', ',
			'output'   => 'string',
			'field'    => 'Valeur'
			),
		$options);
		
		$xpathkey = $xpath.'/NumOrdre';
		if (is_string($options['field'])) $xpath .= '/'.$options['field'];
		if (in_array($options['output'], array('first','last'))) $xpath .= "/.[:{$options['output']}]";

		switch ($options['output']) {
			case 'array':
				return Set::extract($xpath, $data);
				break;		
			case 'ordered':
				return Set::combine($data, $xpathkey, $xpath);
				break;
			case 'string':
			case 'first' :
				$a = Set::extract($xpath, $data);
				$result = '';
				if (!empty($a)) {
					$result .= $options['prepend'];
					$s = '';
					$s .= implode($options['delimiter'], $a);
					if ($options['truncate'] > 0) { 
						$s = $this->text->truncate($s, $options['truncate']);
					}
					$result .= htmlspecialchars($s);
					$result .= $options['append'];
					return $result;
				} else {
					return '';
				}				
			default:
				$result = Set::extract($xpath, $data);	
				break;
		}
	}
	
	function getRedirectUrl($url,$book,$source = 'default'){
		
		if( strpos($url,'http://')  !== false ){
			$redirectActionUrl = $url;
		}else{
			$redirectActionUrl = array(
				'controller'=>'notices',
				'action'=>'doRedirect',
				'IdNotice'=>$this->getId($book),
				'Source' => $source,
				'DetailPageURL' => $url
				);
		}
		return $redirectActionUrl;
	}
	
/**
 * Highlights a given phrase in a text. You can specify any expression in highlighter that
 * may include the \1 expression to include the $phrase found.
 *
 * @param string $text Text to search the phrase in
 * @param string $phrase The phrase that will be searched
 * @param string $highlighter The piece of html with that the phrase will be highlighted
 * @param boolean $considerHtml If true, will ignore any HTML tags, ensuring that only the correct text is highlighted
 * @return string The highlighted text
 * @access public
 */
	function highlight($text, $phrase, $highlighter = '<span class="highlight">\1</span>', $considerHtml = false) {
		if (empty($phrase)) {
			return $text;
		}

		if (is_array($phrase)) {
			$replace = array();
			$with = array();

			foreach ($phrase as $key => $value) {
				$key = $value;
				$value = $highlighter;
				$key = '(' . $key . ')';
				if ($considerHtml) {
					$key = '(?![^<]+>)' . $key . '(?![^<]+>)';
				}
				$replace[] = '|' . $key . '|iu';
				$with[] = empty($value) ? $highlighter : $value;
			}

			return preg_replace($replace, $with, $text);
		} else {
			$phrase = '(' . $phrase . ')';
			if ($considerHtml) {
				$phrase = '(?![^<]+>)' . $phrase . '(?![^<]+>)';
			}

			return preg_replace('|'.$phrase.'|iu', $highlighter, $text);
		}
	}
/**
 * Strips given text of all links (<a href=....)
 *
 * @param string $text Text
 * @return string The text without links
 * @access public
 */
	function stripLinks($text) {
		return preg_replace('|<a\s+[^>]+>|im', '', preg_replace('|<\/a>|im', '', $text));
	}
/**
 * Adds links (<a href=....) to a given text, by finding text that begins with
 * strings like http:// and ftp://.
 *
 * @param string $text Text to add links to
 * @param array $htmlOptions Array of HTML options.
 * @return string The text with links
 * @access public
 */
	function autoLinkUrls($text, $htmlOptions = array()) {
		$options = 'array(';
		foreach ($htmlOptions as $option => $value) {
				$value = var_export($value, true);
				$options .= "'$option' => $value, ";
		}
		$options .= ')';

		$text = preg_replace_callback('#(?<!href="|">)((?:http|https|ftp|nntp)://[^ <]+)#i', create_function('$matches',
			'$Html = new HtmlHelper(); $Html->tags = $Html->loadConfig(); return $Html->link($matches[0], $matches[0],' . $options . ');'), $text);

		return preg_replace_callback('#(?<!href="|">)(?<!http://|https://|ftp://|nntp://)(www\.[^\n\%\ <]+[^<\n\%\,\.\ <])(?<!\))#i',
			create_function('$matches', '$Html = new HtmlHelper(); $Html->tags = $Html->loadConfig(); return $Html->link($matches[0], "http://" . strtolower($matches[0]),' . $options . ');'), $text);
	}
/**
 * Adds email links (<a href="mailto:....) to a given text.
 *
 * @param string $text Text
 * @param array $htmlOptions Array of HTML options.
 * @return string The text with links
 * @access public
 */
	function autoLinkEmails($text, $htmlOptions = array()) {
		$options = 'array(';

		foreach ($htmlOptions as $option => $value) {
			$options .= "'$option' => '$value', ";
		}
		$options .= ')';

		return preg_replace_callback('#([_A-Za-z0-9+-]+(?:\.[_A-Za-z0-9+-]+)*@[A-Za-z0-9-]+(?:\.[A-Za-z0-9-]+)*)#',
						create_function('$matches', '$Html = new HtmlHelper(); $Html->tags = $Html->loadConfig(); return $Html->link($matches[0], "mailto:" . $matches[0],' . $options . ');'), $text);
	}
/**
 * Convert all links and email adresses to HTML links.
 *
 * @param string $text Text
 * @param array $htmlOptions Array of HTML options.
 * @return string The text with links
 * @access public
 */
	function autoLink($text, $htmlOptions = array()) {
		return $this->autoLinkEmails($this->autoLinkUrls($text, $htmlOptions), $htmlOptions);
	}
/**
 * Truncates text.
 *
 * Cuts a string to the length of $length and replaces the last characters
 * with the ending if the text is longer than length.
 *
 * @param string  $text String to truncate.
 * @param integer $length Length of returned string, including ellipsis.
 * @param mixed $ending If string, will be used as Ending and appended to the trimmed string. Can also be an associative array that can contain the last three params of this method.
 * @param boolean $exact If false, $text will not be cut mid-word
 * @param boolean $considerHtml If true, HTML tags would be handled correctly
 * @return string Trimmed string.
 */
	function truncate($text, $length = 100, $ending = '...', $exact = true, $considerHtml = false) {
		if (is_array($ending)) {
			extract($ending);
		}
		if ($considerHtml) {
			if (mb_strlen(preg_replace('/<.*?>/', '', $text)) <= $length) {
				return $text;
			}
			$totalLength = mb_strlen($ending);
			$openTags = array();
			$truncate = '';
			preg_match_all('/(<\/?([\w+]+)[^>]*>)?([^<>]*)/', $text, $tags, PREG_SET_ORDER);
			foreach ($tags as $tag) {
				if (!preg_match('/img|br|input|hr|area|base|basefont|col|frame|isindex|link|meta|param/s', $tag[2])) {
					if (preg_match('/<[\w]+[^>]*>/s', $tag[0])) {
						array_unshift($openTags, $tag[2]);
					} else if (preg_match('/<\/([\w]+)[^>]*>/s', $tag[0], $closeTag)) {
						$pos = array_search($closeTag[1], $openTags);
						if ($pos !== false) {
							array_splice($openTags, $pos, 1);
						}
					}
				}
				$truncate .= $tag[1];

				$contentLength = mb_strlen(preg_replace('/&[0-9a-z]{2,8};|&#[0-9]{1,7};|&#x[0-9a-f]{1,6};/i', ' ', $tag[3]));
				if ($contentLength + $totalLength > $length) {
					$left = $length - $totalLength;
					$entitiesLength = 0;
					if (preg_match_all('/&[0-9a-z]{2,8};|&#[0-9]{1,7};|&#x[0-9a-f]{1,6};/i', $tag[3], $entities, PREG_OFFSET_CAPTURE)) {
						foreach ($entities[0] as $entity) {
							if ($entity[1] + 1 - $entitiesLength <= $left) {
								$left--;
								$entitiesLength += mb_strlen($entity[0]);
							} else {
								break;
							}
						}
					}

					$truncate .= mb_substr($tag[3], 0 , $left + $entitiesLength);
					break;
				} else {
					$truncate .= $tag[3];
					$totalLength += $contentLength;
				}
				if ($totalLength >= $length) {
					break;
				}
			}

		} else {
			if (mb_strlen($text) <= $length) {
				return $text;
			} else {
				$truncate = mb_substr($text, 0, $length - strlen($ending));
			}
		}
		if (!$exact) {
			$spacepos = mb_strrpos($truncate, ' ');
			if (isset($spacepos)) {
				if ($considerHtml) {
					$bits = mb_substr($truncate, $spacepos);
					preg_match_all('/<\/([a-z]+)>/', $bits, $droppedTags, PREG_SET_ORDER);
					if (!empty($droppedTags)) {
						foreach ($droppedTags as $closingTag) {
							if (!in_array($closingTag[1], $openTags)) {
								array_unshift($openTags, $closingTag[1]);
							}
						}
					}
				}
				$truncate = mb_substr($truncate, 0, $spacepos);
			}
		}

		$truncate .= $ending;

		if ($considerHtml) {
			foreach ($openTags as $tag) {
				$truncate .= '</'.$tag.'>';
			}
		}

		return $truncate;
	}
/**
 * Alias for truncate().
 *
 * @see TextHelper::truncate()
 * @access public
 */
	function trim() {
		$args = func_get_args();
		return call_user_func_array(array(&$this, 'truncate'), $args);
	}
/**
 * Extracts an excerpt from the text surrounding the phrase with a number of characters on each side determined by radius.
 *
 * @param string $text String to search the phrase in
 * @param string $phrase Phrase that will be searched for
 * @param integer $radius The amount of characters that will be returned on each side of the founded phrase
 * @param string $ending Ending that will be appended
 * @return string Modified string
 * @access public
 */
	function excerpt($text, $phrase, $radius = 100, $ending = "...") {
		if (empty($text) or empty($phrase)) {
			return $this->truncate($text, $radius * 2, $ending);
		}

		$phraseLen = strlen($phrase);
		if ($radius < $phraseLen) {
			$radius = $phraseLen;
		}

		$pos = strpos(strtolower($text), strtolower($phrase));

		$startPos = 0;
		if ($pos > $radius) {
			$startPos = $pos - $radius;
		}

		$textLen = strlen($text);

		$endPos = $pos + $phraseLen + $radius;
		if ($endPos >= $textLen) {
			$endPos = $textLen;
		}

		$excerpt = substr($text, $startPos, $endPos - $startPos);
		if ($startPos != 0) {
			$excerpt = substr_replace($excerpt, $ending, 0, $phraseLen);
		}

		if ($endPos != $textLen) {
			$excerpt = substr_replace($excerpt, $ending, -$phraseLen);
		}

		return $excerpt;
	}
/**
 * Creates a comma separated list where the last two items are joined with 'and', forming natural English
 *
 * @param array $list The list to be joined
 * @return string
 * @access public
 */
	function toList($list, $and = 'and') {
		$r = '';
		$c = count($list) - 1;
		foreach ($list as $i => $item) {
			$r .= $item;
			if ($c > 0 && $i < $c)
			{
				$r .= ($i < $c - 1 ? ', ' : " {$and} ");
			}
		}
		return $r;
	}
/**
 * Text-to-html parser, similar to Textile or RedCloth, only with a little different syntax.
 *
 * @param string $text String to "flay"
 * @param boolean $allowHtml Set to true if if html is allowed
 * @return string "Flayed" text
 * @access public
 * @todo Change this. We need a real Textile parser.
 * @codeCoverageIgnoreStart
 */
	function flay($text, $allowHtml = false) {
		trigger_error(__('Запрет: расширение the Flay больше не поддерживается и будет удалено в следующей версии программы.', true), E_USER_WARNING);
		if (!class_exists('Flay')) {
			uses('flay');
		}
		return Flay::toHtml($text, false, $allowHtml);
	}
/**
 * @codeCoverageIgnoreEnd
 */
}
?>